package frc.robot.utilities;

import java.util.HashMap;
import java.util.Map;

import edu.wpi.first.wpilibj.smartdashboard.SendableChooser;
import edu.wpi.first.wpilibj2.command.CommandScheduler;
import edu.wpi.first.wpilibj2.command.SubsystemBase;

public class OIProfileManager extends SendableChooser<Runnable> {
    private SubsystemBase[] subsystems;

    private Map<String, Runnable> profiles;

    public OIProfileManager(SubsystemBase... subsystems) {
        super();

        this.subsystems = subsystems;

        profiles = new HashMap<String, Runnable>();

        this.onChange(this::onProfileChanged);
    }

    public void addProfile(String profileName, Runnable setupFunction) {
        if(profiles.containsKey(profileName)) {
            throw new IllegalArgumentException("Profile Name " + profileName + " already exists!");
        }

        profiles.put(profileName, setupFunction);
        this.addOption(profileName, setupFunction);
    }

    // TODO This method has no way to reflect what it changed in the SendableChooser
    public void switchProfile(String profileName) {
        if(!profiles.containsKey(profileName)) {
            throw new IllegalArgumentException("Profile Name " + profileName + " does not exist!");
        }

        clearCurrentConfiguration();

        profiles.get(profileName).run();
    }

    private void clearCurrentConfiguration() {
        CommandScheduler.getInstance().getActiveButtonLoop().clear();
        
        for(SubsystemBase s : subsystems) {
            s.getCurrentCommand().cancel();
            s.removeDefaultCommand();
        }
    }

    private void onProfileChanged(Runnable toRun) {
        clearCurrentConfiguration();

        toRun.run();
    }
}
